# 概述


# 数据预置

可以运行当前目录下的`push.sh`脚本，将默认数据预置到设备。
预置之前需要先修改`push.sh`里面相应的信息以匹配设备信息和版本信息。

## KV

```json
{
    "fota_cldp": "cop2",
    "fota_tourl": "flash2://misc",
    "otaurl": "http://predaily.occ.aliyun-inc.test/api/image/ota/pull",
    "fota_autock": 0,
    "fota_slptm": 10000,
    "fota_rtmout": 10000,
    "fota_wtmout": 5000,
    "versionA": "1.0.0-20211019.1122-R-d1linux",
    "versionB": "1.0.0-20211019.1122-R-d1linux",
    "changelogA": "the init version",
    "changelogB": "the init version"
}

```

## NVRAM

```json
{
    "device_id": "bd8b323204401800a71fb375eb827489",
    "model": "d1linux2",
    "app_version": "1.0.0-20211019.1122-R-d1linux",
    "changelog": "the init version",
}
```

# keystore生成

## RSA公私钥生成

1. PEM私钥格式文件

```bash
openssl genrsa -out key.pem 1024
```

    -out 指定生成文件，此文件包含公钥和私钥两部分，所以即可以加密，也可以解密
    1024 生成密钥的长度

查看私钥文件信息：

```bash
openssl rsa -in key.pem -text
```

2. PEM公钥格式文件

```bash
openssl rsa -in key.pem -pubout -out pubkey.pem
```

    -in 指定输入的密钥文件
    -out 指定提取生成公钥的文件(PEM公钥格式)

3. PEM RSAPublicKey公钥格式文件

```bash
openssl rsa -in key.pem -RSAPublicKey_out -out pubkey.pem
```
    -in 指定输入的密钥文件
    -out 指定提取生成公钥的文件(PEM RSAPublicKey格式)

4. PEM公钥格式文件转BIGNUM文件

```bash
product key -t RSA1024 -k pubkey.pem -o bn_pubkey.txt
```

```bash
product key -t RSA2048 -k pubkey.pem -o bn_pubkey.txt
```

bn_pubkey.txt文件中的N值即为代码中需要用到的公钥。此工程中会把这个值存放在`fota_verify.c`的`g_pubkey_rsa`数组中。
例如：
```c
static uint8_t g_pubkey_rsa[] = {
    0xeb, 0xe3, 0xa4, 0x41, 0x10, 0xd6, 0x25, 0x98, 0xc2, 0x27, 0x8b, 0x36, 0xac, 0xfa, 0xc3, 0x86,
    0x01, 0x21, 0x16, 0x64, 0x6f, 0xf2, 0x37, 0x72, 0xf4, 0xc0, 0x60, 0xab, 0x38, 0x60, 0x47, 0x7a,
    0x94, 0x7f, 0x80, 0x48, 0xa8, 0xeb, 0xa5, 0xf7, 0x1b, 0x9f, 0xf2, 0xc2, 0xf9, 0x39, 0x08, 0xbc,
    0xa9, 0x3e, 0x3c, 0x0c, 0x52, 0x15, 0xb3, 0x57, 0x81, 0x5e, 0x02, 0xf2, 0xd7, 0x7e, 0x04, 0x4c,
    0x6d, 0x93, 0xc4, 0x5d, 0xa3, 0x97, 0x17, 0xa6, 0x83, 0xa5, 0x9c, 0xc4, 0x91, 0xcc, 0x2d, 0x78,
    0xeb, 0x64, 0xbf, 0x05, 0x4d, 0x0b, 0xed, 0x09, 0x4f, 0xd4, 0x2d, 0x46, 0x2d, 0xf0, 0xfc, 0xd0,
    0x03, 0xc0, 0xb7, 0x5c, 0x19, 0x6a, 0x87, 0x11, 0xff, 0xf8, 0xa4, 0x6a, 0x55, 0xa2, 0xf6, 0x25,
    0xb9, 0x00, 0x96, 0xaf, 0x2c, 0x7f, 0x15, 0xb9, 0xe3, 0xe3, 0x24, 0x3d, 0xc3, 0xa7, 0xa4, 0x37,
};
```
